@@ -18,12 +18,17 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
+  along with PulseAudio; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+  USA.
 ***/
 
+typedef struct pa_sink pa_sink;
+typedef struct pa_device_port pa_device_port;
+typedef struct pa_sink_volume_change pa_sink_volume_change;
+
 #include <inttypes.h>
 
-#include <pulsecore/typedefs.h>
 #include <pulse/def.h>
 #include <pulse/format.h>
 #include <pulse/sample.h>
@@ -38,24 +43,32 @@
 #include <pulsecore/asyncmsgq.h>
 #include <pulsecore/msgobject.h>
 #include <pulsecore/rtpoll.h>
-#include <pulsecore/device-port.h>
 #include <pulsecore/card.h>
 #include <pulsecore/queue.h>
 #include <pulsecore/thread-mq.h>
 #include <pulsecore/sink-input.h>
 
-#define PA_MAX_INPUTS_PER_SINK 256
+#define PA_MAX_INPUTS_PER_SINK 32
 
 /* Returns true if sink is linked: registered and accessible from client side. */
-static inline bool PA_SINK_IS_LINKED(pa_sink_state_t x) {
+static inline pa_bool_t PA_SINK_IS_LINKED(pa_sink_state_t x) {
     return x == PA_SINK_RUNNING || x == PA_SINK_IDLE || x == PA_SINK_SUSPENDED;
 }
 
+struct pa_device_port {
+    char *name;
+    char *description;
+
+    unsigned priority;
+
+    /* .. followed by some implementation specific data */
+};
+
+#define PA_DEVICE_PORT_DATA(d) ((void*) ((uint8_t*) d + PA_ALIGN(sizeof(pa_device_port))))
+
 /* A generic definition for void callback functions */
 typedef void(*pa_sink_cb_t)(pa_sink *s);
 
-typedef int (*pa_sink_get_mute_cb_t)(pa_sink *s, bool *mute);
-
 struct pa_sink {
     pa_msgobject parent;
 
@@ -63,12 +76,6 @@ struct pa_sink {
     pa_core *core;
 
     pa_sink_state_t state;
-
-    /* Set in the beginning of pa_sink_unlink() before setting the sink state
-     * to UNLINKED. The purpose is to prevent moving streams to a sink that is
-     * about to be removed. */
-    bool unlink_requested;
-
     pa_sink_flags_t flags;
     pa_suspend_cause_t suspend_cause;
 
@@ -81,8 +88,6 @@ struct pa_sink {
 
     pa_sample_spec sample_spec;
     pa_channel_map channel_map;
-    uint32_t default_sample_rate;
-    uint32_t alternate_sample_rate;
 
     pa_idxset *inputs;
     unsigned n_corked;
@@ -92,22 +97,22 @@ struct pa_sink {
     pa_volume_t base_volume; /* shall be constant */
     unsigned n_volume_steps; /* shall be constant */
 
-    /* Also see http://www.freedesktop.org/wiki/Software/PulseAudio/Documentation/Developer/Volumes/ */
+    /* Also see http://pulseaudio.org/wiki/InternalVolumes */
     pa_cvolume reference_volume; /* The volume exported and taken as reference base for relative sink input volumes */
     pa_cvolume real_volume;      /* The volume that the hardware is configured to  */
     pa_cvolume soft_volume;      /* The internal software volume we apply to all PCM data while it passes through */
 
-    bool muted:1;
+    pa_bool_t muted:1;
 
-    bool refresh_volume:1;
-    bool refresh_muted:1;
-    bool save_port:1;
-    bool save_volume:1;
-    bool save_muted:1;
+    pa_bool_t refresh_volume:1;
+    pa_bool_t refresh_muted:1;
+    pa_bool_t save_port:1;
+    pa_bool_t save_volume:1;
+    pa_bool_t save_muted:1;
 
     /* Saved volume state while we're in passthrough mode */
     pa_cvolume saved_volume;
-    bool saved_save_volume:1;
+    pa_bool_t saved_save_volume:1;
 
     pa_asyncmsgq *asyncmsgq;
 
@@ -115,15 +120,9 @@ struct pa_sink {
 
     pa_hashmap *ports;
     pa_device_port *active_port;
-    pa_atomic_t mixer_dirty;
-
-    /* The latency offset is inherited from the currently active port */
-    int64_t port_latency_offset;
 
     unsigned priority;
 
-    bool set_mute_in_progress;
-
     /* Called when the main loop requests a state change. Called from
      * main loop context. If returns -1 the state change will be
      * inhibited */
@@ -195,24 +194,14 @@ struct pa_sink {
      * set this callback. */
     pa_sink_cb_t write_volume; /* may be NULL */
 
-    /* If the sink mute can change "spontaneously" (i.e. initiated by the sink
-     * implementation, not by someone else calling pa_sink_set_mute()), then
-     * the sink implementation can notify about changed mute either by calling
-     * pa_sink_mute_changed() or by calling pa_sink_get_mute() with
-     * force_refresh=true. If the implementation chooses the latter approach,
-     * it should implement the get_mute callback. Otherwise get_mute can be
-     * NULL.
-     *
-     * This is called when pa_sink_get_mute() is called with
-     * force_refresh=true. This is called from the IO thread if the
-     * PA_SINK_DEFERRED_VOLUME flag is set, otherwise this is called from the
-     * main thread. On success, the implementation is expected to return 0 and
-     * set the mute parameter that is passed as a reference. On failure, the
-     * implementation is expected to return -1.
+    /* Called when the mute setting is queried. A PA_SINK_MESSAGE_GET_MUTE
+     * message will also be sent. Called from IO thread if PA_SINK_DEFERRED_VOLUME
+     * flag is set otherwise from main loop context. If refresh_mute is FALSE
+     * neither this function is called nor a message is sent.
      *
      * You must use the function pa_sink_set_get_mute_callback() to
      * set this callback. */
-    pa_sink_get_mute_cb_t get_mute;
+    pa_sink_cb_t get_mute; /* may be NULL */
 
     /* Called when the mute setting shall be changed. A PA_SINK_MESSAGE_SET_MUTE
      * message will also be sent. Called from IO thread if PA_SINK_DEFERRED_VOLUME
@@ -230,8 +219,8 @@ struct pa_sink {
      * thread context. */
     pa_sink_cb_t update_requested_latency; /* may be NULL */
 
-    /* Called whenever the port shall be changed. Called from IO
-     * thread if deferred volumes are enabled, and main thread otherwise. */
+    /* Called whenever the port shall be changed. Called from main
+     * thread. */
     int (*set_port)(pa_sink *s, pa_device_port *port); /* may be NULL */
 
     /* Called to get the list of formats supported by the sink, sorted
@@ -239,14 +228,10 @@ struct pa_sink {
     pa_idxset* (*get_formats)(pa_sink *s); /* may be NULL */
 
     /* Called to set the list of formats supported by the sink. Can be
-     * NULL if the sink does not support this. Returns true on success,
-     * false otherwise (for example when an unsupportable format is
+     * NULL if the sink does not support this. Returns TRUE on success,
+     * FALSE otherwise (for example when an unsupportable format is
      * set). Makes a copy of the formats passed in. */
-    bool (*set_formats)(pa_sink *s, pa_idxset *formats); /* may be NULL */
-
-    /* Called whenever the sampling frequency shall be changed. Called from
-     * main thread. */
-    int (*update_rate)(pa_sink *s, uint32_t rate);
+    pa_bool_t (*set_formats)(pa_sink *s, pa_idxset *formats); /* may be NULL */
 
     /* Contains copies of the above data so that the real-time worker
      * thread can work without access locking */
@@ -257,12 +242,12 @@ struct pa_sink {
         pa_rtpoll *rtpoll;
 
         pa_cvolume soft_volume;
-        bool soft_muted:1;
+        pa_bool_t soft_muted:1;
 
         /* The requested latency is used for dynamic latency
          * sinks. For fixed latency sinks it is always identical to
          * the fixed_latency. See below. */
-        bool requested_latency_valid:1;
+        pa_bool_t requested_latency_valid:1;
         pa_usec_t requested_latency;
 
         /* The number of bytes streams need to keep around as history to
@@ -275,7 +260,7 @@ struct pa_sink {
 
         /* Maximum of what clients requested to rewind in this cycle */
         size_t rewind_nbytes;
-        bool rewind_requested;
+        pa_bool_t rewind_requested;
 
         /* Both dynamic and fixed latencies will be clamped to this
          * range. */
@@ -287,9 +272,6 @@ struct pa_sink {
          * in changing it */
         pa_usec_t fixed_latency; /* for sinks with PA_SINK_DYNAMIC_LATENCY this is 0 */
 
-        /* This latency offset is a direct copy from s->port_latency_offset */
-        int64_t port_latency_offset;
-
         /* Delayed volume change events are queued here. The events
          * are stored in expiration order. The one expiring next is in
          * the head of the list. */
@@ -327,6 +309,8 @@ typedef enum pa_sink_message {
     PA_SINK_MESSAGE_SET_STATE,
     PA_SINK_MESSAGE_START_MOVE,
     PA_SINK_MESSAGE_FINISH_MOVE,
+    PA_SINK_MESSAGE_ATTACH,
+    PA_SINK_MESSAGE_DETACH,
     PA_SINK_MESSAGE_SET_LATENCY_RANGE,
     PA_SINK_MESSAGE_GET_LATENCY_RANGE,
     PA_SINK_MESSAGE_SET_FIXED_LATENCY,
@@ -337,13 +321,10 @@ typedef enum pa_sink_message {
     PA_SINK_MESSAGE_SET_MAX_REQUEST,
     PA_SINK_MESSAGE_SET_PORT,
     PA_SINK_MESSAGE_UPDATE_VOLUME_AND_MUTE,
-    PA_SINK_MESSAGE_SET_PORT_LATENCY_OFFSET,
     PA_SINK_MESSAGE_MAX
 } pa_sink_message_t;
 
 typedef struct pa_sink_new_data {
-    pa_suspend_cause_t suspend_cause;
-
     char *name;
     pa_proplist *proplist;
 
@@ -356,30 +337,27 @@ typedef struct pa_sink_new_data {
 
     pa_sample_spec sample_spec;
     pa_channel_map channel_map;
-    uint32_t alternate_sample_rate;
     pa_cvolume volume;
-    bool muted:1;
+    pa_bool_t muted :1;
 
-    bool sample_spec_is_set:1;
-    bool channel_map_is_set:1;
-    bool alternate_sample_rate_is_set:1;
-    bool volume_is_set:1;
-    bool muted_is_set:1;
+    pa_bool_t sample_spec_is_set:1;
+    pa_bool_t channel_map_is_set:1;
+    pa_bool_t volume_is_set:1;
+    pa_bool_t muted_is_set:1;
 
-    bool namereg_fail:1;
+    pa_bool_t namereg_fail:1;
 
-    bool save_port:1;
-    bool save_volume:1;
-    bool save_muted:1;
+    pa_bool_t save_port:1;
+    pa_bool_t save_volume:1;
+    pa_bool_t save_muted:1;
 } pa_sink_new_data;
 
 pa_sink_new_data* pa_sink_new_data_init(pa_sink_new_data *data);
 void pa_sink_new_data_set_name(pa_sink_new_data *data, const char *name);
 void pa_sink_new_data_set_sample_spec(pa_sink_new_data *data, const pa_sample_spec *spec);
 void pa_sink_new_data_set_channel_map(pa_sink_new_data *data, const pa_channel_map *map);
-void pa_sink_new_data_set_alternate_sample_rate(pa_sink_new_data *data, const uint32_t alternate_sample_rate);
 void pa_sink_new_data_set_volume(pa_sink_new_data *data, const pa_cvolume *volume);
-void pa_sink_new_data_set_muted(pa_sink_new_data *data, bool mute);
+void pa_sink_new_data_set_muted(pa_sink_new_data *data, pa_bool_t mute);
 void pa_sink_new_data_set_port(pa_sink_new_data *data, const char *port);
 void pa_sink_new_data_done(pa_sink_new_data *data);
 
@@ -393,9 +371,9 @@ pa_sink* pa_sink_new(
 void pa_sink_set_get_volume_callback(pa_sink *s, pa_sink_cb_t cb);
 void pa_sink_set_set_volume_callback(pa_sink *s, pa_sink_cb_t cb);
 void pa_sink_set_write_volume_callback(pa_sink *s, pa_sink_cb_t cb);
-void pa_sink_set_get_mute_callback(pa_sink *s, pa_sink_get_mute_cb_t cb);
+void pa_sink_set_get_mute_callback(pa_sink *s, pa_sink_cb_t cb);
 void pa_sink_set_set_mute_callback(pa_sink *s, pa_sink_cb_t cb);
-void pa_sink_enable_decibel_volume(pa_sink *s, bool enable);
+void pa_sink_enable_decibel_volume(pa_sink *s, pa_bool_t enable);
 
 void pa_sink_put(pa_sink *s);
 void pa_sink_unlink(pa_sink* s);
@@ -409,22 +387,22 @@ void pa_sink_set_max_request(pa_sink *s, size_t max_request);
 void pa_sink_set_latency_range(pa_sink *s, pa_usec_t min_latency, pa_usec_t max_latency);
 void pa_sink_set_fixed_latency(pa_sink *s, pa_usec_t latency);
 
+void pa_sink_detach(pa_sink *s);
+void pa_sink_attach(pa_sink *s);
+
 void pa_sink_set_soft_volume(pa_sink *s, const pa_cvolume *volume);
 void pa_sink_volume_changed(pa_sink *s, const pa_cvolume *new_volume);
-void pa_sink_mute_changed(pa_sink *s, bool new_muted);
+void pa_sink_mute_changed(pa_sink *s, pa_bool_t new_muted);
 
 void pa_sink_update_flags(pa_sink *s, pa_sink_flags_t mask, pa_sink_flags_t value);
 
-bool pa_device_init_description(pa_proplist *p, pa_card *card);
-bool pa_device_init_icon(pa_proplist *p, bool is_sink);
-bool pa_device_init_intended_roles(pa_proplist *p);
+pa_bool_t pa_device_init_description(pa_proplist *p);
+pa_bool_t pa_device_init_icon(pa_proplist *p, pa_bool_t is_sink);
+pa_bool_t pa_device_init_intended_roles(pa_proplist *p);
 unsigned pa_device_init_priority(pa_proplist *p);
 
 /**** May be called by everyone, from main context */
 
-int pa_sink_update_rate(pa_sink *s, uint32_t rate, bool passthrough);
-void pa_sink_set_port_latency_offset(pa_sink *s, int64_t offset);
-
 /* The returned value is supposed to be in the time domain of the sound card! */
 pa_usec_t pa_sink_get_latency(pa_sink *s);
 pa_usec_t pa_sink_get_requested_latency(pa_sink *s);
@@ -435,61 +413,45 @@ size_t pa_sink_get_max_rewind(pa_sink *s);
 size_t pa_sink_get_max_request(pa_sink *s);
 
 int pa_sink_update_status(pa_sink*s);
-int pa_sink_suspend(pa_sink *s, bool suspend, pa_suspend_cause_t cause);
-int pa_sink_suspend_all(pa_core *c, bool suspend, pa_suspend_cause_t cause);
+int pa_sink_suspend(pa_sink *s, pa_bool_t suspend, pa_suspend_cause_t cause);
+int pa_sink_suspend_all(pa_core *c, pa_bool_t suspend, pa_suspend_cause_t cause);
 
 /* Use this instead of checking s->flags & PA_SINK_FLAT_VOLUME directly. */
-bool pa_sink_flat_volume_enabled(pa_sink *s);
+pa_bool_t pa_sink_flat_volume_enabled(pa_sink *s);
 
 /* Get the master sink when sharing volumes */
 pa_sink *pa_sink_get_master(pa_sink *s);
 
-bool pa_sink_is_filter(pa_sink *s);
-
 /* Is the sink in passthrough mode? (that is, is there a passthrough sink input
  * connected to this sink? */
-bool pa_sink_is_passthrough(pa_sink *s);
+pa_bool_t pa_sink_is_passthrough(pa_sink *s);
 /* These should be called when a sink enters/leaves passthrough mode */
 void pa_sink_enter_passthrough(pa_sink *s);
 void pa_sink_leave_passthrough(pa_sink *s);
 
-void pa_sink_set_volume(pa_sink *sink, const pa_cvolume *volume, bool sendmsg, bool save);
-const pa_cvolume *pa_sink_get_volume(pa_sink *sink, bool force_refresh);
+void pa_sink_set_volume(pa_sink *sink, const pa_cvolume *volume, pa_bool_t sendmsg, pa_bool_t save);
+const pa_cvolume *pa_sink_get_volume(pa_sink *sink, pa_bool_t force_refresh);
 
-void pa_sink_set_mute(pa_sink *sink, bool mute, bool save);
-bool pa_sink_get_mute(pa_sink *sink, bool force_refresh);
+void pa_sink_set_mute(pa_sink *sink, pa_bool_t mute, pa_bool_t save);
+pa_bool_t pa_sink_get_mute(pa_sink *sink, pa_bool_t force_refresh);
 
-bool pa_sink_update_proplist(pa_sink *s, pa_update_mode_t mode, pa_proplist *p);
+pa_bool_t pa_sink_update_proplist(pa_sink *s, pa_update_mode_t mode, pa_proplist *p);
 
-int pa_sink_set_port(pa_sink *s, const char *name, bool save);
-void pa_sink_set_mixer_dirty(pa_sink *s, bool is_dirty);
+int pa_sink_set_port(pa_sink *s, const char *name, pa_bool_t save);
 
 unsigned pa_sink_linked_by(pa_sink *s); /* Number of connected streams */
 unsigned pa_sink_used_by(pa_sink *s); /* Number of connected streams which are not corked */
-
-/* Returns how many streams are active that don't allow suspensions. If
- * "ignore_input" or "ignore_output" is non-NULL, that stream is not included
- * in the count (the returned count includes the value from
- * pa_source_check_suspend(), which is called for the monitor source, so that's
- * why "ignore_output" may be relevant). */
-unsigned pa_sink_check_suspend(pa_sink *s, pa_sink_input *ignore_input, pa_source_output *ignore_output);
-
+unsigned pa_sink_check_suspend(pa_sink *s); /* Returns how many streams are active that don't allow suspensions */
 #define pa_sink_get_state(s) ((s)->state)
 
 /* Moves all inputs away, and stores them in pa_queue */
 pa_queue *pa_sink_move_all_start(pa_sink *s, pa_queue *q);
-void pa_sink_move_all_finish(pa_sink *s, pa_queue *q, bool save);
+void pa_sink_move_all_finish(pa_sink *s, pa_queue *q, pa_bool_t save);
 void pa_sink_move_all_fail(pa_queue *q);
 
-/* Returns a copy of the sink formats. TODO: Get rid of this function (or at
- * least get rid of the copying). There's no good reason to copy the formats
- * every time someone wants to know what formats the sink supports. The formats
- * idxset could be stored directly in the pa_sink struct.
- * https://bugs.freedesktop.org/show_bug.cgi?id=71924 */
 pa_idxset* pa_sink_get_formats(pa_sink *s);
-
-bool pa_sink_set_formats(pa_sink *s, pa_idxset *formats);
-bool pa_sink_check_format(pa_sink *s, pa_format_info *f);
+pa_bool_t pa_sink_set_formats(pa_sink *s, pa_idxset *formats);
+pa_bool_t pa_sink_check_format(pa_sink *s, pa_format_info *f);
 pa_idxset* pa_sink_check_formats(pa_sink *s, pa_idxset *in_formats);
 
 /*** To be called exclusively by the sink driver, from IO context */
@@ -516,24 +478,18 @@ void pa_sink_set_fixed_latency_within_thread(pa_sink *s, pa_usec_t latency);
 
 void pa_sink_update_volume_and_mute(pa_sink *s);
 
-bool pa_sink_volume_change_apply(pa_sink *s, pa_usec_t *usec_to_next);
-
-size_t pa_sink_process_input_underruns(pa_sink *s, size_t left_to_play);
+pa_bool_t pa_sink_volume_change_apply(pa_sink *s, pa_usec_t *usec_to_next);
 
 /*** To be called exclusively by sink input drivers, from IO context */
 
 void pa_sink_request_rewind(pa_sink*s, size_t nbytes);
 
-void pa_sink_invalidate_requested_latency(pa_sink *s, bool dynamic);
+void pa_sink_invalidate_requested_latency(pa_sink *s, pa_bool_t dynamic);
 
 pa_usec_t pa_sink_get_latency_within_thread(pa_sink *s);
 
-/* Called from the main thread, from sink-input.c only. The normal way to set
- * the sink reference volume is to call pa_sink_set_volume(), but the flat
- * volume logic in sink-input.c needs also a function that doesn't do all the
- * extra stuff that pa_sink_set_volume() does. This function simply sets
- * s->reference_volume and fires change notifications. */
-void pa_sink_set_reference_volume_direct(pa_sink *s, const pa_cvolume *volume);
+pa_device_port *pa_device_port_new(const char *name, const char *description, size_t extra);
+void pa_device_port_free(pa_device_port *p);
 
 /* Verify that we called in IO context (aka 'thread context), or that
  * the sink is not yet set up, i.e. the thread not set up yet. See
